
using Autofac;
using Autofac.Extensions.DependencyInjection;
using AutoMapper.EquivalencyExpression;
using FellowOakDicom;
using FellowOakDicom.Imaging;
using FellowOakDicom.Imaging.NativeCodec;
using FellowOakDicom.Network;
using IRaCIS.Core.Infra.EFCore;
using IRaCIS.Core.SCP;
using IRaCIS.Core.SCP.Filter;
using IRaCIS.Core.SCP.Service;
using MassTransit;
using MassTransit.NewIdProviders;
using Microsoft.AspNetCore.HttpOverrides;
using Microsoft.Extensions.DependencyInjection;
using Panda.DynamicWebApi;
using Serilog;
using Serilog.Events;
using System.Runtime.InteropServices;


//ļΪ׼ urlȡֵ(дݲļ˾ͲҪݻ)
var config = new ConfigurationBuilder()
            .AddEnvironmentVariables()
            .Build();

var enviromentName = config["ASPNETCORE_ENVIRONMENT"];

var builder = WebApplication.CreateBuilder(new WebApplicationOptions
{
    EnvironmentName = enviromentName
});



#region 

NewId.SetProcessIdProvider(new CurrentProcessIdProvider());

builder.Configuration.AddJsonFile("appsettings.json", false, true)
                     .AddJsonFile($"appsettings.{enviromentName}.json", false, true);
builder.Host
            .UseServiceProviderFactory(new AutofacServiceProviderFactory())
            .ConfigureContainer<ContainerBuilder>(containerBuilder =>
            {
                containerBuilder.RegisterModule<AutofacModuleSetup>();
            })
            .UseSerilog();
#endregion

#region ÷
var _configuration = builder.Configuration;

//
builder.Services.AddHealthChecks();

//ػ
builder.Services.AddJsonLocalization(options => options.ResourcesPath = "Resources");


// 쳣ͳһ֤JsonлáַͳһTrim()
builder.Services.AddControllers(options =>
{
    options.Filters.Add<ModelActionFilter>();
    options.Filters.Add<ProjectExceptionFilter>();
    options.Filters.Add<UnitOfWorkFilter>();


})
    .AddNewtonsoftJsonSetup(builder.Services); // NewtonsoftJson л  


builder.Services.AddOptions().Configure<AliyunOSSOptions>(_configuration.GetSection("AliyunOSS"));
builder.Services.AddOptions().Configure<ObjectStoreServiceOptions>(_configuration.GetSection("ObjectStoreService"));
builder.Services.AddOptions().Configure<DicomSCPServiceOption>(_configuration.GetSection("DicomSCPServiceConfig"));


//̬WebApi + UnifiedApiResultFilter  ʡ
//̬webApi     ĿǰڵΨһСapiϷϵĶ̬AOPʧЧ    ӵòӰ
builder.Services
           .AddDynamicWebApi(dynamicWebApiOption =>
           {
               //Ĭ api
               dynamicWebApiOption.DefaultApiPrefix = "";
               //ĸСд
               dynamicWebApiOption.GetRestFulActionName = (actionName) => char.ToLower(actionName[0]) + actionName.Substring(1);
               //ɾ Service׺
               dynamicWebApiOption.RemoveControllerPostfixes.Add("Service");

           });

//AutoMapper
builder.Services.AddAutoMapper(automapper =>
{

    automapper.AddCollectionMappers();


}, typeof(BaseService).Assembly);

//EF ORM  QueryWithNoLock
builder.Services.AddEFSetup(_configuration);

builder.Services.AddMediator(cfg =>
{

});


//תͷ  ȡʵIP 
builder.Services.Configure<ForwardedHeadersOptions>(options =>
{
    options.ForwardedHeaders =
        ForwardedHeaders.XForwardedFor | ForwardedHeaders.XForwardedProto;
});


builder.Services.AddFellowOakDicom().AddTranscoderManager<NativeTranscoderManager>()
              //.AddTranscoderManager<FellowOakDicom.Imaging.NativeCodec.NativeTranscoderManager>()
              .AddImageManager<ImageSharpImageManager>();



#endregion

// Add services to the container.

builder.Services.AddControllers();
// Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
builder.Services.AddEndpointsApiExplorer();
builder.Services.AddSwaggerGen();

var app = builder.Build();

// Configure the HTTP request pipeline.
//if (app.Environment.IsDevelopment())
//{
app.UseSwagger();
app.UseSwaggerUI();
//}

app.UseAuthorization();

app.MapControllers();

#region ־


Log.Logger = new LoggerConfiguration()
     //.MinimumLevel.Information()
     .MinimumLevel.Override("Microsoft", LogEventLevel.Warning)
    .WriteTo.Console()
    .WriteTo.File($"{AppContext.BaseDirectory}Serilogs/.log", rollingInterval: RollingInterval.Day)
    .CreateLogger();

#endregion


#region л ƽ̨

Log.Logger.Warning($"ǰ{enviromentName}");

if (RuntimeInformation.IsOSPlatform(OSPlatform.Windows))
{
    Log.Logger.Warning($"ǰƽ̨windows");
}
else if (RuntimeInformation.IsOSPlatform(OSPlatform.Linux))
{
    Log.Logger.Warning($"ǰƽ̨linux");
}
else
{
    Log.Logger.Warning($"ǰƽ̨OSX  or FreeBSD");
}

#endregion

DicomSetupBuilder.UseServiceProvider(app.Services);

var logger = app.Services.GetService<Microsoft.Extensions.Logging.ILogger<Program>>();

var server = DicomServerFactory.Create<CStoreSCPService>(_configuration.GetSection("DicomSCPServiceConfig").GetValue<int>("ServerPort"), userState: app.Services, logger: logger);


app.Run();
